import java.util.Iterator;
import java.util.Map;

class AliasCommand extends Command
{
    AliasCommand ()
    {
        super ("alias", "\034F15\035/alias [name [command]]\034C\035\n\nCreate a new alias with the given name for a command. " +
                       "Multiple commands may also be supplied, separated by \";\". If no command is specified, the alias " +
                       "will instead be printed. If no name is specified, all aliases will be printed.\n" +
                       "The following tokens may be used to refer to the alias' arguments:\n" +
                       "$1 through $9 - the first through ninth arguments, respectively\n" +
                       "$@ - all arguments not referred to by the above\n" +
                       "$* - all arguments supplied to the alias\n" +
                       "If none of the above tokens are used, the alias will behave as if \" $*\" were placed at the end.\n" +
                       "The following tokens may be used to insert control characters into an alias:\n" +
                       "$$ - expands to \"$\"\n" +
                       "$; - expands to \";\"\n");
    }

    static int usedArgs (String alias)
    {
        int usedArgs = 0, 
            index = alias.indexOf ('$');
        while (index >= 0)
        {
            ++ index;
            if (index >= alias.length ())
                break;
            char c = alias.charAt (index);
            if (c == '*')
                usedArgs |= 1 << 10;
            else
            if (c == '@')
                usedArgs |= 1 << 11;
            else
            if (c >= '1' && c <= '9')
                usedArgs |= 1 << (c - '0'); 
            index = alias.indexOf ('$', index + 1);
        }
        return usedArgs;
    }

    static String expandAlias (String alias, String [] args)
    {
        int usedArgs = usedArgs (alias);
        StringBuffer expansion = new StringBuffer ();
        int index = 0;
        while (index < alias.length ())
        {
            int nextIndex = alias.indexOf ('$', index);
            if (nextIndex < 0)
                nextIndex = alias.length ();
            expansion.append (alias.substring (index, nextIndex).replace (';', '\n'));
            ++ nextIndex;
            if (nextIndex < alias.length ())
            {
                char c = alias.charAt (nextIndex);
                if (c == '*')
                {
                    for (int arg = 1; arg < args.length; ++ arg)
                    {
                        expansion.append (args [arg]);
                        if (arg + 1 < args.length)
                            expansion.append (' ');
                    }
                }
                else
                if (c == '@')
                {
                    boolean separate = false;
                    for (int arg = 1; arg < args.length; ++ arg)
                    {
                        if ((usedArgs & (1 << arg)) == 0)
                        {
                            if (separate)
                                expansion.append (' ');
                            expansion.append (args [arg]);
                            separate = true;
                        }
                    }
                }
                else
                if (c >= '1' && c <= '9')
                {
                    int arg = c - '0';
                    if (arg >= 1 && arg < args.length)
                        expansion.append (args [arg]);
                }
                else
                {
                    expansion.append (c);
                }
            }
            else
            if (nextIndex == alias.length ())
                expansion.append ('$');
            index = nextIndex + 1;
        }
        if (usedArgs == 0)
        {
            for (int arg = 1; arg < args.length; ++ arg)
            {
                expansion.append (' ');
                expansion.append (args [arg]);
            }
        }
        return expansion.toString ();
    }

    boolean execute (Terminal terminal, String [] args)
    {
        if (args.length == 1)
        {
            if (terminal.aliases.isEmpty ())
                terminal.addText ("There are no aliases.\n");
            else
            {
                Iterator i = terminal.aliases.entrySet ().iterator ();
                boolean separate = false;
                while (i.hasNext ())
                {
                    Map.Entry entry = (Map.Entry) i.next ();
                    String name = (String) entry.getKey (),
                           alias = (String) entry.getValue ();
                    terminal.addText ("\034H" + alias + "\035\034F6\035" + name + "\034C\035: \034F15\035" + alias + "\034C\035\n");
                }
            }
        }
        else
        if (args.length == 2)
        {
            if (! terminal.aliases.containsKey (args [1]))
                terminal.addText ("There is no alias \"\034F9\035" + args [1] + "\034C\035\".\n");
            else
                terminal.addText (args [1] + ": " + (String) terminal.aliases.get (args [1]) + "\n");
        }
        else
        {
            StringBuffer buffer = new StringBuffer ();
            for (int arg = 2; arg < args.length; ++ arg)
            {
                buffer.append (args [arg]);
                if (arg + 1 < args.length)
                    buffer.append (' ');
            }
            String alias = buffer.toString ();
            terminal.aliases.put (args [1], alias);
            terminal.addText ("Aliased \"\034H" + alias + "\035\034F6\035" + args [1] + "\034C\035\" to \"\034F15\035" + alias + "\034C\035\".\n"); 
        }
        return true;
    }    
}

