import java.awt.GraphicsEnvironment;

class FontCommand extends Command
{
    FontCommand ()
    {
        super ("font", "\034F15\035/font [name [size]]\034C\035\n\n" +
                        "Set the terminal's font name and/or size. " +
                        "If no size is specified, the size will be inherited from the current font. " +
                        "If no name is specified, the current font name and size will instead be printed.\n" +
                        "The following fonts are available:\n");
    }

    String help ()
    {
        StringBuffer message = new StringBuffer (help);
        String [] fonts = GraphicsEnvironment.getLocalGraphicsEnvironment ().getAvailableFontFamilyNames ();
        for (int i = 0; i < fonts.length; ++ i)
        {
            if (i > 0)
                message.append (i + 1 < fonts.length ? ", " : ", and ");
            message.append ("\034H/font \"" + fonts [i] + "\"\035\034F6\035" + fonts [i] + "\034C\035");
        }
        message.append (".\n");
        return message.toString ();
    }
        
    boolean execute (Terminal terminal, String [] args)
    {
        if (args.length >= 2)
        {
            String name = args [1];
            int size;
            if (args.length >= 3)
                size = Math.max (Integer.parseInt (args [2]), 1);
            else
                size = terminal.font.getSize ();
            terminal.setFont (name, size);
        }
        String family = terminal.font.getFamily ();
        int size = terminal.font.getSize ();
        terminal.addText ("The current font is \034F15\035" + family + "\034C\035 at size \034F15\035" + (new Integer (size)).toString () + "\034C\035.\n");
        return true;
    }    
}

