import javax.swing.JComponent;
import java.awt.AlphaComposite;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Image;
import java.util.TreeMap;

class Map extends JComponent
{
    int columns, rows;
    Terrain [] terrains;
    TreeMap entities;
    Terrain [] grid;
    LinkedList [] contents;
    LinkedList overlays;
    UI ui;
    
    Map (UI u)
    {
        ui = u;
        terrains = new Terrain [128];
        for (int i = 0; i < terrains.length; ++ i)
            terrains [i] = new Terrain (i);
        entities = new TreeMap ();

        setGrid (2, 2, new char [] { 0, 1, 2, 3 });
        addOverlay ("overlay.png", 0, 1, 2, 1);
        addEntity ("me", "avatar.png");
        moveEntity ("me", 0, 0);
    }

    void setGrid (int c, int r, char [] g)
    {
        columns = c;
        rows = r;
        grid = new Terrain [columns * rows];
        for (int i = 0; i < columns * rows; ++ i)
            grid [i] = terrains [g [i]];
        contents = new LinkedList [columns * rows];
        overlays = new LinkedList ();
    }

    boolean addOverlay (String name, int x, int y, int w, int h)
    {
        if (x < 0 || x >= columns || y < 0 || y >= rows ||
            w < 1 || w > columns || h < 1 || h > columns)
            return false;
        overlays.addLast (new Overlay (ui.cacheImage (name), x, y, w, h));
        return true;
    }

    boolean removeOverlays ()
    {
        if (overlays.isEmpty ())
            return false;
        overlays.clear ();
        return true;
    }

    boolean addEntity (String name, String imageName)
    {
        if (entities.containsKey (name))
            return false;
        entities.put (name, new Entity (name, ui.cacheImage (imageName)));
        return true;
    }

    boolean removeEntity (String name)
    {
        if (! entities.containsKey (name))
            return false;
        entities.remove (name);
        return true;
    }

    boolean moveEntity (String name, int column, int row)
    {
        if (column < 0 || column >= columns || row < 0 || row >= rows ||
            ! entities.containsKey (name))
            return false;
        Entity entity = (Entity) entities.get (name);
        LinkedList list = contents [row * columns + column];
        if (list == null)
        {
            list = new LinkedList ();
            contents [row * columns + column] = list;
        }
        entity.remove ();
        list.addLast (entity);
        return true;
    }

    public void paint (Graphics g)
    {
        Graphics2D g2 = (Graphics2D) g;
        g2.setComposite (AlphaComposite.SrcAtop);

        int tileWidth = getWidth () / columns,
            tileHeight = getHeight () / rows;
        for (int i = 0, y = 0; y < rows; ++ y)
        {
            for (int x = 0; x < columns; ++ x, ++ i)
            {
                grid [i].draw (g2, this, x * tileWidth, y * tileHeight, tileWidth, tileHeight);
            }
        }
        for (Link link = overlays.first (); link != overlays.end (); link = link.nextLink)
        {
            Overlay overlay = (Overlay) link;
            overlay.draw (g2, this, tileWidth, tileHeight);
        }
        for (int i = 0, y = 0; y < rows; ++ y)
        {
            for (int x = 0; x < columns; ++ x, ++ i)
            {
                LinkedList list = contents [i];
                if (list != null)
                {
                    for (Link link = list.first (); link != list.end (); link = link.nextLink)
                    {
                        Entity entity = (Entity) link;
                        entity.draw (g2, this, x * tileWidth, y * tileHeight, tileWidth, tileHeight);
                    }
                }
            }
        }
    }
}

