import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollBar;
import javax.swing.JTextField;
import javax.swing.Spring;
import javax.swing.SpringLayout;
import javax.swing.SwingConstants;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Image;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseWheelListener;
import java.awt.event.MouseWheelEvent;
import java.util.TreeMap;

class UI extends JFrame
    implements ActionListener, AdjustmentListener, ComponentListener, MouseWheelListener 
{
    JScrollBar scrollBar;
    JTextField inputArea;
    JPanel buttonPanel;
    TreeMap buttons;
    TreeMap imageCache;
    Map map;
    Terminal terminal;

    
    UI (String name)
    {
        super (name);
        setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
    
        scrollBar = new JScrollBar (SwingConstants.VERTICAL);
        inputArea = new JTextField ();
        buttonPanel = new JPanel (new FlowLayout (FlowLayout.LEFT));
        buttons = new TreeMap ();
        imageCache = new TreeMap ();
        map = new Map (this);
        terminal = new Terminal (this, 1000);

        SpringLayout layout = new SpringLayout ();
        getContentPane ().setLayout (layout);
       
        getContentPane ().add (map);
        getContentPane ().add (terminal);
        getContentPane ().add (scrollBar);
        getContentPane ().add (buttonPanel);
        getContentPane ().add (inputArea);
         
        SpringLayout.Constraints termCons = layout.getConstraints (terminal);
        termCons.setX (Spring.constant (0));
        termCons.setY (Spring.constant (0));

        SpringLayout.Constraints mapCons = layout.getConstraints (map);
        mapCons.setY (Spring.constant (0));
        mapCons.setHeight (termCons.getConstraint (SpringLayout.SOUTH));

        SpringLayout.Constraints sbCons = layout.getConstraints (scrollBar);
        sbCons.setX (termCons.getConstraint (SpringLayout.EAST));
        sbCons.setY (Spring.constant (0));
        sbCons.setWidth (Spring.constant (2 * terminal.cellWidth + 2 * terminal.BORDER));
        sbCons.setHeight (termCons.getHeight ());
        mapCons.setX (sbCons.getConstraint (SpringLayout.EAST));

        SpringLayout.Constraints bpCons = layout.getConstraints (buttonPanel);
        bpCons.setX (Spring.constant (0));
        bpCons.setY (termCons.getConstraint (SpringLayout.SOUTH));
        bpCons.setHeight (Spring.constant (5 * terminal.cellHeight / 2));
        bpCons.setWidth (Spring.sum (mapCons.getWidth (), Spring.sum (termCons.getWidth (), sbCons.getWidth ())));

        SpringLayout.Constraints inCons = layout.getConstraints (inputArea);
        inCons.setX (Spring.constant (0));
        inCons.setY (bpCons.getConstraint (SpringLayout.SOUTH));
        inCons.setHeight (Spring.constant (terminal.cellHeight + 2 * terminal.BORDER));
        inCons.setWidth (bpCons.getWidth ());

        SpringLayout.Constraints paneCons = layout.getConstraints (getContentPane ());
        paneCons.setConstraint (SpringLayout.EAST,
                                mapCons.getConstraint (SpringLayout.EAST));
        paneCons.setConstraint (SpringLayout.SOUTH,
                                inCons.getConstraint (SpringLayout.SOUTH));

        getContentPane ().addComponentListener (this);
        getContentPane ().addMouseWheelListener (this);
        scrollBar.addAdjustmentListener (this);
        inputArea.addActionListener (this);
        
        pack ();

        map.repaint ();

        addButton ("help", "/help");
        addButton ("aliases", "/alias");
        addButton ("copy", "/copy");
        addButton ("paste", "/paste");
        addButton ("quit", "/quit");

        setVisible (true);

        inputArea.requestFocus ();
    }
   
    void run ()
    {
        terminal.run ();
    }

    Image cacheImage (String name)
    {
        if (! imageCache.containsKey (name))
            imageCache.put (name, getToolkit ().createImage (name));
        return (Image) imageCache.get (name);
    }
        
    void updateScrollBar (int extent, int limit)
    {
        int oldValue = Math.max (scrollBar.getMaximum () - scrollBar.getValue (), extent),
            newMax = Math.max (limit, extent);
        scrollBar.setValues (Math.max (newMax - oldValue, 0), extent, 0, newMax);
    }
        
    boolean addButton (String name, String action)
    {
        if (buttons.containsKey (name))
            return false;

        JButton button = new JButton (name);
        button.setFocusable (false);
        button.setActionCommand (action);
        button.addActionListener (this);
        buttonPanel.add (button);
        buttons.put (name, button);
        buttonPanel.doLayout ();
        buttonPanel.repaint ();
        return true;
    }
 
    boolean removeButton (String name)
    {
        if (! buttons.containsKey (name))
            return false;

        JButton button = (JButton) buttons.remove (name);
        buttonPanel.remove (button);
        button.removeActionListener (this);
        buttonPanel.doLayout ();
        buttonPanel.repaint ();
        return true;
    }

    public Dimension getPreferredSize ()
    {
        return new Dimension (640, 480);
    }

// ComponentListener
    public void componentHidden (ComponentEvent event)
    {
    }

    public void componentMoved (ComponentEvent event)
    {
    }

    public void componentShown (ComponentEvent event)
    {
    }

    public void componentResized (ComponentEvent event)
    {
        terminal.updateCells ();
    }

// ActionListener
    public void actionPerformed (ActionEvent event)
    {
        if (event.getSource () == inputArea)
        {
            terminal.input (inputArea.getText ());
            inputArea.setText ("");
        }
        else
            terminal.input (event.getActionCommand ());
    }

// AdjustmentListener
    public void adjustmentValueChanged (AdjustmentEvent event)
    {
        terminal.setScrollOffset (scrollBar.getMaximum () - scrollBar.getVisibleAmount () - scrollBar.getValue ());
    }

// MouseWheelListener
    public void mouseWheelMoved (MouseWheelEvent event)
    {
        if (event.getScrollType () == MouseWheelEvent.WHEEL_UNIT_SCROLL)
            scrollBar.setValue (scrollBar.getValue () + event.getWheelRotation ());
    }
}
        
